@@ -581,7 +581,7 @@ getSpectralEnvelope = function(
 #' argument (eg for a low/high/bandpass filter).
 #'
 #' Algorithm: converts input from a time series (time domain) to a spectrogram
-#' (frequency domain) through short-term Fourier transform (STFT), multiples by
+#' (frequency domain) through short-time Fourier transform (STFT), multiples by
 #' the spectral filter containing the specified formants, and transforms back to
 #' a time series via inverse STFT. This is a subroutine for voice synthesis in
 #' \code{\link{soundgen}}, but it can also be applied to a recording.
@@ -652,6 +652,22 @@ getSpectralEnvelope = function(
 #' spectrogram(s_denoised, 16000)
 #' # playme(s_denoised)
 #'
+#' # If neither formants nor spectralEnvelope are defined, only lipRad has an effect
+#' # For ex., we can boost low frequencies by 6 dB/oct
+#' noise = runif(8000)
+#' noise1 = addFormants(noise, 16000, lipRad = -6)
+#' seewave::meanspec(noise1, f = 16000, dB = 'max0')
+#'
+#' # Arbitrary spectra can be defined with spectralEnvelope. For ex., we can
+#' # have a flat spectrum up to 2 kHz (Nyquist / 4) and -3 dB/kHz above:
+#' freqs = seq(0, 16000 / 2, length.out = 100)
+#' n = length(freqs)
+#' idx = (n / 4):n
+#' sp_dB = c(rep(0, n / 4 - 1), (freqs[idx] - freqs[idx[1]]) / 1000 * (-3))
+#' plot(freqs, sp_dB, type = 'b')
+#' noise2 = addFormants(noise, 16000, lipRad = 0, spectralEnvelope = 10 ^ (sp_dB / 20))
+#' seewave::meanspec(noise2, f = 16000, dB = 'max0')
+#'
 #' ## Use the spectral envelope of an existing recording (bleating of a sheep)
 #' # (see also the same example with noise as source in ?generateNoise)
 #' # (NB: this can also be achieved with a single call to transplantFormants)
@@ -698,7 +714,7 @@ getSpectralEnvelope = function(
 addFormants = function(
   x,
   samplingRate = NULL,
-  formants,
+  formants = NULL,
   spectralEnvelope = NULL,
   zFun = NULL,
   action = c('add', 'remove')[1],
@@ -1115,15 +1131,13 @@ transplantFormants = function(donor,
   }
   for (i in 1:ncol(spec_recipient)) {
     abs_s = abs(spec_recipient[, i])
-    # plot(abs_s, type = 'l')
+    # plot(log(abs_s), type = 'l')
     cor_coef = flatEnv(
       abs_s, samplingRate = 1, method = 'peak',
+      dynamicRange = dynamicRange,
       windowLength_points = freqWindow_bins) / abs_s
     # plot(Re(spec_recipient[, i]) * cor_coef, type = 'l')
-    spec_recipient[, i] = complex(
-      real = Re(spec_recipient[, i]) * cor_coef,
-      imaginary = Im(spec_recipient[, i])
-    )
+    spec_recipient[, i] = spec_recipient[, i] * cor_coef
     # plot(abs(spec_recipient[, i]), type = 'l')
   }
 
@@ -1139,6 +1153,10 @@ transplantFormants = function(donor,
 
   # Multiply the spectrograms and reconstruct the audio
   spec_recipient_new = spec_recipient * spec_donor_rightDim
+  # image(log(abs(t(spec_recipient))))
+  # image(log(t(spec_donor_rightDim)))
+  # image(log(abs(t(spec_recipient_new))))
+
   recipient_new = as.numeric(
     seewave::istft(
       spec_recipient_new,
